import resmushit
import pytest
from resmushit import Resmushit
import os
from resmushit.exceptions import InvalidImageExtensionError

FIXTURES = os.path.dirname(os.path.abspath(__file__)) + "/fixtures"


def test_from_path():
    assert (
        resmushit.from_path(
            image_path=FIXTURES+"/sample.jpg",
            output_dir=FIXTURES+"/optimized"
        )
        == None
    )


def test_from_path_without_path():
    with pytest.raises(ValueError):
        resmushit.from_path()



def test_from_path_bytes_output():
    assert isinstance(
        resmushit.from_path(
            image_path=FIXTURES+"/sample.jpg",
            save=False
        ),
        bytes,
    )


import resmushit
import pytest
from resmushit import Resmushit
import os
from resmushit.exceptions import InvalidImageExtensionError, FileTooLargeError

FIXTURES = os.path.dirname(os.path.abspath(__file__)) + "/fixtures"


def test_max_file_size():
    with pytest.raises(FileTooLargeError):
        resmushit.from_path(
            image_path=FIXTURES + "/sample_gt_5_mb.jpg", output_dir=FIXTURES + "/optimized"
        )



import pytest
from resmushit import Resmushit
import os
from resmushit.exceptions import InvalidImageExtensionError
from resmushit.validator import Validator

FIXTURES = os.path.dirname(os.path.abspath(__file__)) + "/fixtures"


def test_webp_extension():
    with pytest.raises(InvalidImageExtensionError):
        Resmushit(image_path=FIXTURES + "/sample.webp", _type="file").optimize()


def test_jpg_extension():
    imagebytes = Validator(path=FIXTURES + "/sample.jpg", _type='file', max_file_size=Resmushit._filesize_limit())._open_image_from_path()
    ext = Validator(path=FIXTURES + "/sample.jpg", _type='file', max_file_size=Resmushit._filesize_limit())._find_image_extension(imagebytes=imagebytes)
    assert ext == 'jpg'
 


class FileTooLargeError(Exception):
    pass


class InvalidImageExtensionError(Exception):
    pass


class ImageURLNotFoundException(Exception):
    pass


class ImagePathNotFoundException(Exception):
    pass


class ImageExtensionNotFoundException(Exception):
    pass



from datetime import datetime
from sys import stdout

class Log:

    def __init__(self, quiet_mode) -> None:
        self.quiet_mode = quiet_mode

    def log(self, message):
        if not self.quiet_mode:
            print(f"{datetime.now().strftime('%Y-%m-%d %H:%M:%S')} {message}")



import json
import os
from urllib3 import PoolManager
from .log import Log
from .validator import Validator
from typing import Union

class Resmushit:
    """
    Wrapper for Resmush.it API

    Args:
        image_url (str): The url of image.
        image_path (str): The path of image
        quality (int): Quality at which image is going to be optimized.
        output_dir (str): Location for output image.
        preserve_exif (bool): Preserve EXIF data in the file after optimization.
        preserve_filename (bool): Optimized image will prefix 'optimized-' before image name.
        quiet_mode (bool): Run in quiet mode when True

    Returns:
        bytes or None: if save=True, image is saved into mentioned output directory
                       if save=False, image bytes are returned
    ## Usage:
        `import resmushit`

        `resmushit.from_path(image_path='image.png', quality=95)`

        `buffer = resmushit.from_url(image_url="https://ps.w.org/resmushit-image-optimizer/assets/icon-128x128.png", quality=95, save=False)`

    """

    __MAX_FILESIZE = 5 * 1024 * 1024
    __API_URL: str = "http://api.resmush.it"

    def __init__(
        self,
        image_url: str = None,
        image_path: str = None,
        _type: str = None,
        quality: int = 92,
        output_dir: str = ".",
        preserve_exif: bool = False,
        preserve_filename: bool = False,
        quiet_mode: bool = False,
    ) -> None:
        if image_path is not None and image_url is not None:
            raise ValueError("Either image_path or image_url should be passed")
        elif image_url is not None:
            self.image_url = image_url
            self.image_path = None
        elif image_path is not None:
            self.image_path = image_path
            self.image_url = None
        else:
            raise ValueError("image_path or image_url param is required")
        self._type = _type
        self.quality = quality
        self.output_dir = output_dir
        self.preserve_exif = preserve_exif
        self.preserve_filename = preserve_filename
        self.quiet_mode = quiet_mode
        self._response = None
        self.__logger = Log(quiet_mode=quiet_mode)

    @classmethod
    def _filesize_limit(cls):
        return cls.__MAX_FILESIZE

    def __call_api(self) -> None:
        try:
            self.__logger.log(
                message=f"Initializing image optimization with quality factor: {self.quality}%"
            )
            self.__logger.log(
                message=f"Sending picture {self.filename}.{self.extension} to api..."
            )
            http = PoolManager()
            r = http.request(
                "POST",
                Resmushit.__API_URL
                + f"/?qlty={self.quality}&exif={self.preserve_exif}",
                fields={"files": (f"{self.filename}.{self.extension}", self.imagebytes)},
            )
            self._response = json.loads(r.data.decode("utf-8"))
            self.__logger.log(
                message=f"File optimized by {self._response.get('percent',0)}% (from {self._response.get('src_size',0)//1024}KB to {self._response.get('dest_size',0)//1024}KB). Retrieving..."
            )
        except Exception as e:
            raise Exception(f"Error Occurred: {str(e)}")

    def __get_dest_url(self) -> str:
        return self._response.get("dest")

    def __download_image(self, dest_url) -> bytes:
        try:
            http = PoolManager()
            r = http.request("GET", dest_url)
            return r.data
        except Exception as e:
            raise Exception(f"{e}")

    def __save_image(self, imagebytes) -> bytes:
        with open(
            os.path.join(
                self.output_dir,
                f"{'' if self.preserve_filename else 'optimized-'}{self.filename}"
                +"."+ f"{self.extension}",
            ),
            "wb",
        ) as f:
            f.write(imagebytes)

    def optimize(self, save: bool = True) -> Union[None, bytes]:
        self.imagebytes, self.filename, self.extension = Validator(
            path=self.image_url or self.image_path,
            max_file_size=Resmushit.__MAX_FILESIZE,
            _type=self._type,
        ).validate()
        self.__logger.log(f"Processing: {self.filename}")
        self.__call_api()

        dest_url = self.__get_dest_url()
        optimized_imagebytes = self.__download_image(dest_url=dest_url)
        if save:
            self.__save_image(imagebytes=optimized_imagebytes)
        else:
            return optimized_imagebytes



import resmushit
import pytest
from resmushit import Resmushit
import os
from resmushit.exceptions import InvalidImageExtensionError

FIXTURES = os.path.dirname(os.path.abspath(__file__)) + "/fixtures"

def test_from_url():
    assert (
        resmushit.from_url(
            image_url="https://letsenhance.io/static/8f5e523ee6b2479e26ecc91b9c25261e/1015f/MainAfter.jpg",
            output_dir=FIXTURES+"/optimized"
        )
        == None
    )


def test_from_url_without_url():
    with pytest.raises(ValueError):
        resmushit.from_url()



def test_from_url_bytes_output():
    assert isinstance(
        resmushit.from_url(
            "https://letsenhance.io/static/8f5e523ee6b2479e26ecc91b9c25261e/1015f/MainAfter.jpg",
            save=False,
        ),
        bytes,
    )



import pathlib
import setuptools

setuptools.setup(
    name="resmushit",
    version="1.0.1",
    description="A wrapper for resmush.it API",
    long_description=pathlib.Path("README.md").read_text(encoding='utf-8'),
    long_description_content_type="text/markdown",
    url="https://resmush.it/",
    author="Hemant Sah",
    author_email="hemantsah18@gmail.com",
    maintainer="Hemant Sah",
    download_url="https://github.com/hemansah/resmushit",
    license="MIT",
    project_urls={
        "Homepage":"https://resmush.it/",
        "Documentation":"https://github.com/hemansah/resmushit/blob/master/README.md",
        "Repository":"https://github.com/hemansah/resmushit/",
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.6",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Scientific/Engineering :: Image Processing",
        "Topic :: Utilities",
    ],
    python_requires=">=3.6,<=3.12",
    install_requires=['urllib3', 'filetype'],
    packages=setuptools.find_packages(exclude=["*.tests", "*.tests.*", "tests.*", "tests"]),
    include_package_data=True,
)



import os
from urllib.parse import urlparse
from urllib3 import PoolManager
from .exceptions import (FileTooLargeError, 
                         InvalidImageExtensionError, 
                         ImageExtensionNotFoundException)
import filetype


class Validator:
    """
        Standard Validator for reSmushit
    """
    def __init__(self, path, max_file_size, _type):
        self.path = path
        self._type = _type
        self.max_file_size = max_file_size
        self.max_file_size_kb = max_file_size // 1024
        self.__ALLOWED_FILETYPES = ("png", "jpg", "jpeg", "gif", "bmp", "tiff", "tiff")

    def __check_url_validity(self):
        if urlparse(self.path).scheme in ["http", "https", "ftp"]:
            return True
        else:
            raise ValueError(f"URL scheme required: {self.path}")

    def __file_exists(self):
        if os.path.exists(self.path):
            return True
        raise FileNotFoundError(f"{self.path}")

    def _open_image_from_url(self):
        http = PoolManager()
        r = http.request("GET", url=self.path)
        return r.data

    def _open_image_from_path(self):
        data = ""
        with open(self.path, "rb") as f:
            data = f.read()
        return data

    def _find_image_extension(self, imagebytes):
        kind = filetype.guess(obj=imagebytes)
        if kind.extension:
            self.extension = kind.extension
            return kind.extension
        raise ImageExtensionNotFoundException(f"'{kind.extension}' image extension found")


    def __check_file_size(self, imagebytes):

        if len(imagebytes) > self.max_file_size:
            raise FileTooLargeError(
                f"Max allowed file size: {self.max_file_size_kb}KB."
            )

    def __get_file_name(self):
        filename, _ = os.path.splitext(os.path.basename(urlparse(self.path).path))
        return filename

    # def __get_file_extension(self):
    #     _, extension = os.path.splitext(os.path.basename(urlparse(self.path).path))
    #     return extension

    def __check_allowed_filetypes(self, imagebytes):
        
        if not self.extension in self.__ALLOWED_FILETYPES:
            raise InvalidImageExtensionError(
                f"'{self.extension}' extension type is not allowed.\nAllowed types: {self.__ALLOWED_FILETYPES}"
            )

    def validate(self):
        if self._type == "url":
            self.__check_url_validity()
            imagebytes = self._open_image_from_url()
            self._find_image_extension(imagebytes=imagebytes)
            self.__check_allowed_filetypes(imagebytes=imagebytes)
            self.__check_file_size(imagebytes=imagebytes)
            filename = self.__get_file_name()
            # fileext = self.__get_file_extension()
            return  imagebytes, filename, self.extension

        if self._type == "file":
            self.__file_exists()
            imagebytes = self._open_image_from_path()
            self._find_image_extension(imagebytes=imagebytes)
            self.__check_allowed_filetypes(imagebytes=imagebytes)
            filename = self.__get_file_name()
            # fileext = self.__get_file_extension()
            self.__check_file_size(imagebytes=imagebytes)
            return imagebytes, filename, self.extension



from .resmushit import Resmushit
from typing import Union

def from_url(image_url=None, _type="url", quality=92, output_dir=".", preserve_exif=False, preserve_filename=False, quiet_mode=False, save=True) -> Union[None, bytes]:
    resmushit = Resmushit(image_url=image_url, _type=_type, image_path=None, quality=quality, output_dir=output_dir, preserve_exif=preserve_exif, preserve_filename=preserve_filename, quiet_mode=quiet_mode)
    return resmushit.optimize(save=save)


def from_path(image_path=None, _type="file", quality=92, output_dir=".", preserve_exif=False, preserve_filename=False, quiet_mode=False, save=True) -> Union[None, bytes]:
    resmushit = Resmushit(image_path=image_path, _type=_type, image_url=None, quality=quality, output_dir=output_dir, preserve_exif=preserve_exif, preserve_filename=preserve_filename, quiet_mode=quiet_mode)
    return resmushit.optimize(save=save)
